@@ -15,7 +15,7 @@
 """
 
 import _thread as thread
-from time import sleep
+import time
 
 from django.conf import settings
 from django.views import generic
@@ -27,6 +27,7 @@ try:
     from openstack import config as occ
 except ImportError:
     from os_client_config import config as occ
+from openstack import exceptions
 
 from openstack_dashboard.api import neutron
 from openstack_dashboard.api.rest import urls
@@ -111,7 +112,7 @@ def poll_loadbalancer_status(request, loadbalancer_id, callback,
     interval = conf.HORIZON_CONFIG['ajax_poll_interval'] / 1000.0
     status = from_state
     while status == from_state:
-        sleep(interval)
+        time.sleep(interval)
         conn = _get_sdk_connection(request)
         lb = conn.load_balancer.get_load_balancer(loadbalancer_id)
         status = lb.provisioning_status
@@ -123,6 +124,68 @@ def poll_loadbalancer_status(request, loadbalancer_id, callback,
         callback(request, **kwargs)
 
 
+def _retry_on_conflict(conn, func, *args, retry_timeout=120, **kwargs):
+    load_balancer_getter = kwargs.pop('load_balancer_getter')
+    resource_id = kwargs.pop('resource_id')
+
+    interval = conf.HORIZON_CONFIG['ajax_poll_interval'] / 1000.0
+
+    load_balancer_id = load_balancer_getter(conn, resource_id)
+
+    start = time.time()
+    while time.time() - start < retry_timeout:
+        lb = conn.load_balancer.get_load_balancer(load_balancer_id)
+        if lb.provisioning_status == 'PENDING_UPDATE':
+            time.sleep(interval)
+            continue
+
+        try:
+            func(*args, **kwargs)
+        except exceptions.ConflictException:
+            # Still catching 409/Conflict as there might have multiple threads
+            # waiting for a non-PENDING provisioning state
+            time.sleep(interval)
+            continue
+        break
+
+
+def retry_on_conflict(conn, func, *args, retry_timeout=120, **kwargs):
+    load_balancer_getter = kwargs.pop('load_balancer_getter')
+    resource_id = kwargs.pop('resource_id')
+
+    try:
+        func(*args, **kwargs)
+    except exceptions.ConflictException:
+        thread.start_new_thread(
+            _retry_on_conflict,
+            (conn, func, *args),
+            {'retry_timeout': retry_timeout,
+             'load_balancer_getter': load_balancer_getter,
+             'resource_id': resource_id,
+             **kwargs})
+
+
+def listener_get_load_balancer_id(conn, listener_id):
+    listener = conn.load_balancer.get_listener(listener_id)
+    return listener.load_balancers[0]['id']
+
+
+def l7_policy_get_load_balancer_id(conn, l7_policy_id):
+    l7_policy = conn.load_balancer.get_l7_policy(l7_policy_id)
+    listener = conn.load_balancer.get_listener(l7_policy.listener_id)
+    return listener.load_balancers[0]['id']
+
+
+def pool_get_load_balancer_id(conn, pool_id):
+    pool = conn.load_balancer.get_pool(pool_id)
+    return pool.loadbalancers[0]['id']
+
+
+def health_monitor_get_load_balancer_id(conn, health_monitor_id):
+    pool_id = conn.load_balancer.get_health_monitor(health_monitor_id)
+    return pool_get_load_balancer_id(conn, pool_id)
+
+
 def create_loadbalancer(request):
     data = request.DATA
 
@@ -185,6 +248,9 @@ def create_listener(request, **kwargs):
         timeout_member_connect=data['listener'].get('timeout_member_connect'),
         timeout_member_data=data['listener'].get('timeout_member_data'),
         timeout_tcp_inspect=data['listener'].get('timeout_tcp_inspect'),
+        allowed_cidrs=data['listener'].get('allowed_cidrs'),
+        # Replace empty string by None (uses default tls cipher string)
+        tls_ciphers=data['listener'].get('tls_ciphers') or None,
     )
 
     if data.get('pool'):
@@ -251,7 +317,10 @@ def create_pool(request, **kwargs):
         loadbalancer_id=kwargs['loadbalancer_id'],
         name=data['pool'].get('name'),
         description=data['pool'].get('description'),
-        admin_state_up=data['pool'].get('admin_state_up')
+        admin_state_up=data['pool'].get('admin_state_up'),
+        tls_enabled=data['pool'].get('tls_enabled'),
+        # Replace empty string by None (uses default tls cipher string)
+        tls_ciphers=data['pool'].get('tls_ciphers') or None,
     )
 
     if data.get('members'):
@@ -457,6 +526,9 @@ def update_listener(request, **kwargs):
         timeout_member_connect=data['listener'].get('timeout_member_connect'),
         timeout_member_data=data['listener'].get('timeout_member_data'),
         timeout_tcp_inspect=data['listener'].get('timeout_tcp_inspect'),
+        allowed_cidrs=data['listener'].get('allowed_cidrs'),
+        # Replace empty string by None (uses default tls cipher string)
+        tls_ciphers=data['listener'].get('tls_ciphers') or None,
     )
 
     if data.get('pool'):
@@ -525,7 +597,10 @@ def update_pool(request, **kwargs):
         session_persistence=data['pool'].get('session_persistence'),
         name=data['pool'].get('name'),
         description=data['pool'].get('description'),
-        admin_state_up=data['pool'].get('admin_state_up')
+        admin_state_up=data['pool'].get('admin_state_up'),
+        tls_enabled=data['pool'].get('tls_enabled'),
+        # Replace empty string by None (uses default tls cipher string)
+        tls_ciphers=data['pool'].get('tls_ciphers') or None,
     )
 
     # Assemble the lists of member id's to add and remove, if any exist
@@ -556,19 +631,27 @@ def update_monitor(request, **kwargs):
     """
     data = request.DATA
     monitor_id = data['monitor']['id']
+    hm_type = data['monitor']['type']
 
     conn = _get_sdk_connection(request)
+    healthmonitor_kwargs = {
+        'delay': data['monitor'].get('delay'),
+        'timeout': data['monitor'].get('timeout'),
+        'max_retries': data['monitor'].get('max_retries'),
+        'max_retries_down': data['monitor'].get('max_retries_down'),
+        'admin_state_up': data['monitor'].get('admin_state_up'),
+        'name': data['monitor'].get('name')
+    }
+    if hm_type in ('HTTP', 'HTTPS'):
+        healthmonitor_kwargs.update({
+            'http_method': data['monitor'].get('http_method'),
+            'url_path': data['monitor'].get('url_path'),
+            'expected_codes': data['monitor'].get('expected_codes')
+            })
+
     healthmonitor = conn.load_balancer.update_health_monitor(
         monitor_id,
-        delay=data['monitor'].get('delay'),
-        timeout=data['monitor'].get('timeout'),
-        max_retries=data['monitor'].get('max_retries'),
-        max_retries_down=data['monitor'].get('max_retries_down'),
-        http_method=data['monitor'].get('http_method'),
-        url_path=data['monitor'].get('url_path'),
-        expected_codes=data['monitor'].get('expected_codes'),
-        admin_state_up=data['monitor'].get('admin_state_up'),
-        name=data['monitor'].get('name')
+        **healthmonitor_kwargs
     )
 
     return _get_sdk_object_dict(healthmonitor)
@@ -838,7 +921,11 @@ class Listener(generic.View):
         http://localhost/api/lbaas/listeners/cc758c90-3d98-4ea1-af44-aab405c9c915
         """
         conn = _get_sdk_connection(request)
-        conn.load_balancer.delete_listener(listener_id, ignore_missing=True)
+        retry_on_conflict(
+            conn, conn.load_balancer.delete_listener,
+            listener_id, ignore_missing=True,
+            load_balancer_getter=listener_get_load_balancer_id,
+            resource_id=listener_id)
 
 
 @urls.register
@@ -921,7 +1008,11 @@ class L7Policy(generic.View):
         http://localhost/api/lbaas/l7policies/cc758c90-3d98-4ea1-af44-aab405c9c915
         """
         conn = _get_sdk_connection(request)
-        conn.load_balancer.delete_l7_policy(l7_policy_id)
+        retry_on_conflict(
+            conn, conn.load_balancer.delete_l7_policy,
+            l7_policy_id,
+            load_balancer_getter=l7_policy_get_load_balancer_id,
+            resource_id=l7_policy_id)
 
 
 @urls.register
@@ -980,7 +1071,11 @@ class L7Rule(generic.View):
     def delete(self, request, l7_rule_id, l7_policy_id):
         """Delete a specific l7 rule."""
         conn = _get_sdk_connection(request)
-        conn.load_balancer.delete_l7_rule(l7_rule_id, l7_policy_id)
+        retry_on_conflict(
+            conn, conn.load_balancer.delete_l7_rule,
+            l7_rule_id, l7_policy_id,
+            load_balancer_getter=l7_policy_get_load_balancer_id,
+            resource_id=l7_policy_id)
 
 
 @urls.register
@@ -1094,7 +1189,11 @@ class Pool(generic.View):
         http://localhost/api/lbaas/pools/cc758c90-3d98-4ea1-af44-aab405c9c915
         """
         conn = _get_sdk_connection(request)
-        conn.load_balancer.delete_pool(pool_id)
+        retry_on_conflict(
+            conn, conn.load_balancer.delete_pool,
+            pool_id,
+            load_balancer_getter=pool_get_load_balancer_id,
+            resource_id=pool_id)
 
 
 @urls.register
@@ -1160,7 +1259,11 @@ class Member(generic.View):
 
         """
         conn = _get_sdk_connection(request)
-        conn.load_balancer.delete_member(member_id, pool_id)
+        retry_on_conflict(
+            conn, conn.load_balancer.delete_member,
+            member_id, pool_id,
+            load_balancer_getter=pool_get_load_balancer_id,
+            resource_id=pool_id)
 
     @rest_utils.ajax()
     def put(self, request, member_id, pool_id):
@@ -1250,8 +1353,12 @@ class HealthMonitor(generic.View):
         http://localhost/api/lbaas/healthmonitors/cc758c90-3d98-4ea1-af44-aab405c9c915
         """
         conn = _get_sdk_connection(request)
-        conn.load_balancer.delete_health_monitor(health_monitor_id,
-                                                 ignore_missing=True)
+        retry_on_conflict(
+            conn, conn.load_balancer.delete_health_monitor,
+            health_monitor_id,
+            ignore_missing=True,
+            load_balancer_getter=health_monitor_get_load_balancer_id,
+            resource_id=health_monitor_id)
 
     @rest_utils.ajax()
     def put(self, request, health_monitor_id):
