@@ -27,7 +27,7 @@
     'horizon.app.core.openstack-service-api.neutron',
     'horizon.app.core.openstack-service-api.nova',
     'horizon.app.core.openstack-service-api.lbaasv2',
-    'horizon.app.core.openstack-service-api.barbican',
+    'horizon.app.core.openstack-service-api.octavia-barbican',
     'horizon.app.core.openstack-service-api.serviceCatalog',
     'horizon.framework.util.i18n.gettext'
   ];
@@ -45,7 +45,7 @@
    * @param neutronAPI The neutron service API.
    * @param novaAPI The nova service API.
    * @param lbaasv2API The LBaaS V2 service API.
-   * @param barbicanAPI The barbican service API.
+   * @param octaviaBarbicanAPI The octavia barbican service API.
    * @param serviceCatalog The keystone service catalog API.
    * @param gettext The horizon gettext function for translation.
    * @returns The model service for the create load balancer workflow.
@@ -56,7 +56,7 @@
     neutronAPI,
     novaAPI,
     lbaasv2API,
-    barbicanAPI,
+    octaviaBarbicanAPI,
     serviceCatalog,
     gettext
   ) {
@@ -86,16 +86,16 @@
       members: [],
       networks: {},
       flavors: {},
-      listenerProtocols: ['HTTP', 'TCP', 'TERMINATED_HTTPS', 'HTTPS', 'UDP'],
+      listenerProtocols: ['HTTP', 'TCP', 'TERMINATED_HTTPS', 'HTTPS', 'UDP', 'SCTP'],
       availability_zones: {},
       l7policyActions: ['REJECT', 'REDIRECT_TO_URL', 'REDIRECT_TO_POOL'],
       l7ruleTypes: ['HOST_NAME', 'PATH', 'FILE_TYPE', 'HEADER', 'COOKIE'],
       l7ruleCompareTypes: ['REGEX', 'EQUAL_TO', 'STARTS_WITH', 'ENDS_WITH', 'CONTAINS'],
       l7ruleFileTypeCompareTypes: ['REGEX', 'EQUAL_TO'],
-      poolProtocols: ['HTTP', 'HTTPS', 'PROXY', 'PROXYV2', 'TCP', 'UDP'],
+      poolProtocols: ['HTTP', 'HTTPS', 'PROXY', 'PROXYV2', 'TCP', 'UDP', 'SCTP'],
       methods: ['LEAST_CONNECTIONS', 'ROUND_ROBIN', 'SOURCE_IP'],
       types: ['SOURCE_IP', 'HTTP_COOKIE', 'APP_COOKIE'],
-      monitorTypes: ['HTTP', 'HTTPS', 'PING', 'TCP', 'TLS-HELLO', 'UDP-CONNECT'],
+      monitorTypes: ['HTTP', 'HTTPS', 'PING', 'TCP', 'TLS-HELLO', 'UDP-CONNECT', 'SCTP'],
       monitorMethods: ['GET', 'HEAD', 'POST', 'PUT', 'DELETE',
                        'TRACE', 'OPTIONS', 'PATCH', 'CONNECT'],
       certificates: [],
@@ -137,7 +137,10 @@
       model.context = {
         resource: resource,
         id: id,
-        submit: null
+        submit: null,
+        create_listener: true,
+        create_pool: true,
+        create_monitor: true
       };
 
       model.certificates = [];
@@ -169,7 +172,9 @@
           timeout_client_data: 50000,
           timeout_member_connect: 5000,
           timeout_member_data: 50000,
-          timeout_tcp_inspect: 0
+          timeout_tcp_inspect: 0,
+          allowed_cidrs: null,
+          tls_ciphers: null
         },
         l7policy: {
           id: null,
@@ -200,7 +205,9 @@
             type: null,
             cookie_name: null
           },
-          admin_state_up: true
+          admin_state_up: true,
+          tls_enabled: false,
+          tls_ciphers: null
         },
         monitor: {
           id: null,
@@ -515,6 +522,7 @@
         if (finalSpec.listener.protocol !== 'TERMINATED_HTTPS') {
           // Remove certificate containers if not using TERMINATED_HTTPS
           delete finalSpec.certificates;
+          delete finalSpec.listener.tls_ciphers;
         } else {
           var containers = [];
           angular.forEach(finalSpec.certificates, function(cert) {
@@ -535,6 +543,9 @@
         // otherwise has to match it.
         var protocol = finalSpec.listener ? finalSpec.listener.protocol : finalSpec.pool.protocol;
         finalSpec.pool.protocol = protocol === 'TERMINATED_HTTPS' ? 'HTTP' : protocol;
+        if (!finalSpec.pool.tls_enabled) {
+          delete finalSpec.pool.tls_ciphers;
+        }
         if (angular.isObject(finalSpec.pool.session_persistence)) {
           if (!finalSpec.pool.session_persistence.type) {
             finalSpec.pool.session_persistence = null;
@@ -729,7 +740,7 @@
                 model.spec.certificates.push(cert);
               });
             });
-          });
+          }, certificatesError);
           $('#wizard-side-nav ul li:last').show();
         }
       }
@@ -801,6 +812,8 @@
       spec.timeout_member_connect = listener.timeout_member_connect;
       spec.timeout_member_data = listener.timeout_member_data;
       spec.timeout_tcp_inspect = listener.timeout_tcp_inspect;
+      spec.allowed_cidrs = listener.allowed_cidrs;
+      spec.tls_ciphers = listener.tls_ciphers;
     }
 
     function setL7PolicySpec(l7policy) {
@@ -835,6 +848,8 @@
       spec.lb_algorithm = pool.lb_algorithm;
       spec.admin_state_up = pool.admin_state_up;
       spec.session_persistence = pool.session_persistence;
+      spec.tls_enabled = pool.tls_enabled;
+      spec.tls_ciphers = pool.tls_ciphers;
     }
 
     function setMembersSpec(membersList) {
@@ -880,8 +895,8 @@
 
     function prepareCertificates() {
       return $q.all([
-        barbicanAPI.getCertificates(true),
-        barbicanAPI.getSecrets(true)
+        octaviaBarbicanAPI.getCertificates(true),
+        octaviaBarbicanAPI.getSecrets(true)
       ]).then(onGetCertificates, certificatesError);
     }
 
